@@ -36,12 +36,12 @@ L.Util.setFromQueryString = function (options, name) {
 
 L.Util.setBooleanFromQueryString = function (options, name) {
     var value = L.Util.queryString(name);
-    if (typeof value !== "undefined") {
-        options[name] = value == "1" || value == "true";
+    if (typeof value !== 'undefined') {
+        options[name] = value == '1' || value == 'true';
     }
 };
 L.Util.escapeHTML = function (s) {
-    s = s? s.toString() : "";
+    s = s? s.toString() : '';
     return s.replace(/</gm, '&lt;');
 };
 L.Util.toHTML = function (r) {
@@ -55,10 +55,10 @@ L.Util.toHTML = function (r) {
 
 
     // headings and hr
-    r = r.replace(/^### (.*)=*/gm, '<h5>$1</h5>');
-    r = r.replace(/^## (.*)=*/gm, '<h4>$1</h4>');
-    r = r.replace(/^# (.*)=*/gm, '<h3>$1</h3>');
-    r = r.replace(/^[-*][-*][-*]+/gm, '<hr>');
+    r = r.replace(/^### (.*)/gm, '<h5>$1</h5>');
+    r = r.replace(/^## (.*)/gm, '<h4>$1</h4>');
+    r = r.replace(/^# (.*)/gm, '<h3>$1</h3>');
+    r = r.replace(/^---/gm, '<hr>');
 
     // bold, italics
     r = r.replace(/\*\*(.*?)\*\*/g, '<strong>$1</strong>');
@@ -72,7 +72,7 @@ L.Util.toHTML = function (r) {
     // links
     r = r.replace(/(\[\[http)/g, '[[h_t_t_p');  // Escape for avoiding clash between [[http://xxx]] and http://xxx
     r = r.replace(/({{http)/g, '{{h_t_t_p');
-    r = r.replace(/(https?[^ \)\n]*)/g, '<a target="_blank" href="$1">$1</a>');
+    r = r.replace(/(https?:[^ \)\n]*)/g, '<a target="_blank" href="$1">$1</a>');
     r = r.replace(/\[\[(h_t_t_ps?:[^\]|]*?)\]\]/g, '<a target="_blank" href="$1">$1</a>');
     r = r.replace(/\[\[(h_t_t_ps?:[^|]*?)\|(.*?)\]\]/g, '<a target="_blank" href="$1">$2</a>');
     r = r.replace(/\[\[([^\]|]*?)\]\]/g, '<a href="$1">$1</a>');
@@ -90,12 +90,12 @@ L.Util.toHTML = function (r) {
     r = r.replace(/(h_t_t_p)/g, 'http');
 
     // Preserver line breaks
-    if (newline) r = r.replace(new RegExp(newline, 'g'), '<br>' + newline);
+    if (newline) r = r.replace(new RegExp(newline + '(?=[^]+)', 'g'), '<br>' + newline);
 
     return r;
 };
 L.Util.isObject = function (what) {
-    return typeof what === "object" && what !== null;
+    return typeof what === 'object' && what !== null;
 };
 L.Util.latLngsForGeoJSON = function (latlngs) {
     coords = [];
@@ -121,7 +121,7 @@ L.Util.detectFileType = function (f) {
     if (ext('.gpx')) return 'gpx';
     if (ext('.geojson') || ext('.json')) return 'geojson';
     if (f.type === 'text/csv' || ext('.csv') || ext('.tsv') || ext('.dsv')) {
-        return 'dsv';
+        return 'csv';
     }
     if (ext('.xml')) return 'osm';
 };
@@ -130,6 +130,18 @@ L.Util.usableOption = function (options, option) {
     return typeof options[option] !== 'undefined' && options[option] !== '' && options[option] !== null;
 };
 
+L.Util.greedyTemplate = function (str, data, fallback) {
+    // Don't throw error if some key is missing
+    return str.replace(/\{ *([\w_]+) *\}/g, function (str, key) {
+        var value = data[key];
+        if (value === undefined) {
+            value = fallback || '';
+        }
+        return value;
+    });
+};
+
+
 
 L.DomUtil.add = function (tagName, className, container, content) {
     var el = L.DomUtil.create(tagName, className, container);
@@ -276,17 +288,17 @@ L.Storage.Help = L.Class.extend({
     },
 
     resolve: function (name) {
-        return typeof this[name] === "function" ? this[name]() : this[name];
+        return typeof this[name] === 'function' ? this[name]() : this[name];
     },
 
     button: function (container, entries) {
         var helpButton = L.DomUtil.create('a', 'storage-help-button', container);
-        helpButton.href = "#";
+        helpButton.href = '#';
         if (entries) {
             L.DomEvent
                 .on(helpButton, 'click', L.DomEvent.stop)
                 .on(helpButton, 'click', function (e) {
-                    var args = typeof entries === "string"? [entries] : entries;
+                    var args = typeof entries === 'string'? [entries] : entries;
                     this.show.apply(this, args);
                 }, this);
         }
@@ -357,12 +369,19 @@ L.Storage.Help = L.Class.extend({
         L.DomUtil.add('li', '', elements, L._('Link with text: [[http://example.com|text of the link]]'));
         L.DomUtil.add('li', '', elements, L._('Image: {{http://image.url.com}}'));
         L.DomUtil.add('li', '', elements, L._('Image with custom width (in px): {{http://image.url.com|width}}'));
-        L.DomUtil.add('li', '', elements, L._('Iframe: {{{http://image.url.com}}}'));
-        L.DomUtil.add('li', '', elements, L._('Iframe with custom height (in px): {{{http://image.url.com|height}}}'));
+        L.DomUtil.add('li', '', elements, L._('Iframe: {{{http://iframe.url.com}}}'));
+        L.DomUtil.add('li', '', elements, L._('Iframe with custom height (in px): {{{http://iframe.url.com|height}}}'));
         L.DomUtil.add('li', '', elements, L._('--- for an horizontal rule'));
         return container;
     },
 
+    dynamicProperties: function () {
+        var container = L.DomUtil.create('div');
+        L.DomUtil.add('h3', '', container, L._('Dynamic properties'));
+        L.DomUtil.add('p', '', container, L._('Use placeholders with feature properties between brackets, eg. &#123;name&#125;, they will be dynamically replaced by the corresponding values.'));
+        return container;
+    },
+
     formatURL: L._('Supported variables that will be dynamically replaced') + ': {bbox}, {lat}, {lng}, {zoom}, {east}, {north}..., {left}, {top}...',
     formatIconURL: L._('You can use feature properties as variables: ex.: with "http://myserver.org/images/{name}.png", the {name} variable will be replaced by the "name" value of each markers.')
 
